<?php

namespace Drupal\graphql_vertex_ai\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * GraphQL Vertex AI settings.
 */
class GraphqlVertexAiSettingsForm extends ConfigFormBase {

  /**
   * Constructs a GraphqlVertexAiSettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typedConfigManager
   *   The typed config manager.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $dateFormatter
   *   The date formatter service.
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    TypedConfigManagerInterface $typedConfigManager,
    protected DateFormatterInterface $dateFormatter,
  ) {
    parent::__construct($configFactory, $typedConfigManager);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'graphql_vertex_ai_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['graphql_vertex_ai.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['key'] = [
      '#title' => 'Key',
      '#type' => 'key_select',
      '#description' => $this->t('Service account authentication_multivalue key.'),
      '#key_filters' => [
        'type' => 'authentication_multivalue',
      ],
      '#config_target' => 'graphql_vertex_ai.settings:key',
    ];

    $form['project_id'] = [
      '#title' => $this->t('Project ID'),
      '#type' => 'textfield',
      '#description' => $this->t('Google project ID.'),
      '#config_target' => 'graphql_vertex_ai.settings:project_id',
    ];

    $form['location'] = [
      '#title' => $this->t('Location'),
      '#type' => 'textfield',
      '#description' => $this->t('Google project location.'),
      '#config_target' => 'graphql_vertex_ai.settings:location',
    ];

    $form['data_store_id'] = [
      '#title' => $this->t('Data Store ID'),
      '#type' => 'textfield',
      '#description' => $this->t('Google dataStore ID.'),
      '#config_target' => 'graphql_vertex_ai.settings:data_store_id',
    ];

    $form['serving_config_id'] = [
      '#title' => $this->t('Serving Config ID'),
      '#type' => 'textfield',
      '#description' => $this->t('Google servingConfig ID. Usually the ID of your Agent in Agent Builder.'),
      '#config_target' => 'graphql_vertex_ai.settings:serving_config_id',
    ];

    $form['facet_limit'] = [
      '#title' => $this->t('Facet limit'),
      '#type' => 'number',
      '#description' => $this->t('The maximum number of values to be returned for each facet.'),
      '#config_target' => 'graphql_vertex_ai.settings:facet_limit',
    ];

    // Identical options to the ones for block caching.
    // @see \Drupal\Core\Block\BlockBase::buildConfigurationForm()
    $period = [0, 60, 180, 300, 600, 900, 1800, 2700, 3600, 10800, 21600, 32400, 43200, 86400];
    $period = array_map([$this->dateFormatter, 'formatInterval'], array_combine($period, $period));
    $form['cache_max_age'] = [
      '#title' => $this->t('Cache max-age'),
      '#type' => 'select',
      '#description' => $this->t('The cache lifetime of Vertex AI requests. It is recommended to use a value higher than 0 to protect your site from DDoS.'),
      '#options' => $period,
      '#config_target' => 'graphql_vertex_ai.settings:cache_max_age',
    ];

    return parent::buildForm($form, $form_state);
  }

}
